import requests, json
from datetime import timedelta
SYNO_API_QC = 'http://@.quickconnect.to/Serv.php'
SYNO_API_QC_PING = '/webman/pingpong.cgi'
SYNO_API_QC_PING_TIMEOUT = 3
        
def get_server(name):
    qc_url = SYNO_API_QC.replace('@', name)
    res = None
    
    for vid in ['dsm_portal', 'dsm_portal_https']:
        urls = set()
        req = [{"version":1, "command":"get_server_info", "id":vid, "serverID":name}]

        data = requests.post(qc_url, data = json.dumps(req, ensure_ascii=False)).json()[0]
        if 'errinfo' in data:
            continue
        proto = 'http' + ('s' if vid.endswith('s') else '')

        if data['service']['pingpong'] == 'CONNECTED':
            port = data['service']['ext_port']
            if not port:
                port = data['service']['port']
            # Local address. Yes, just that straightforward
            urls.add(proto + '://' + data['server']['interface'][0]['ip'] + ':' + str(port))

            urls.add(proto + '://' + data['server']['external']['ip'] + ':' + str(port))
        min = timedelta(hours=1)

        for url in urls:
            try:
                ping_req = requests.get(url + SYNO_API_QC_PING, timeout = SYNO_API_QC_PING_TIMEOUT)
                if ping_req.elapsed < min:
                    ping = ping_req.json()
                    if ping['success']:
                        min = ping_req.elapsed
                        res = url
            except:
                pass
    return res


if __name__ == '__main__':
    print(get_server('casabobuk'))



import aiohttp, asyncio, async_timeout, json

async def fetch_json(session, url, js = None, timeout = 60):
    headers = {'accept': 'application/json'}
    with async_timeout.timeout(timeout):
        if not js:
            async with session.get(url, headers=headers) as resp:
                return json.loads(await resp.text())
        else:
            async with session.post(url, headers=headers,data=json.dumps(js, ensure_ascii=False)) as resp:
                return json.loads(await resp.text())
            
async def post_plain(session, url, js, timeout = 60):
    headers = {'accept': 'application/json'}
    with async_timeout.timeout(timeout):
        async with session.post(url, headers=headers, data=js) as resp:
            return json.loads(await resp.text())
            



from pygments import highlight
from pygments.lexers import PythonLexer
from pygments.formatters import Terminal256Formatter
from pprint import pformat

def pp(obj):
    print(highlight(pformat(obj), PythonLexer(), Terminal256Formatter()))



import aiohttp, asyncio, async_timeout, json
from .utils import *

SYNO_API_QC = 'http://@.quickconnect.to/Serv.php'
SYNO_API_QC_PING = '/webman/pingpong.cgi'
SYNO_API_QC_PING_TIMEOUT = 3
        
async def get_server(name):
    qc_url = SYNO_API_QC.replace('@', name)
    
    async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(verify_ssl=False)) as http:
        for vid in ['dsm_portal', 'dsm_portal_https']:
            urls = set()
            req = [{"version":1, "command":"get_server_info", "id":vid, "serverID":name}]

            data = (await fetch_json(http, qc_url, js = req))[0]
            if 'errinfo' in data:
                continue
            proto = 'http' + ('s' if vid.endswith('s') else '')

            if data['service']['pingpong'] == 'CONNECTED':
                port = data['service']['ext_port']
                if not port:
                    port = data['service']['port']
                # Local address. Yes, just that straightforward
                urls.add(proto + '://' + data['server']['interface'][0]['ip'] + ':' + str(port))

                urls.add(proto + '://' + data['server']['external']['ip'] + ':' + str(port))
            for url in urls:
                try:
                    ping = await fetch_json(http, url + SYNO_API_QC_PING, timeout = SYNO_API_QC_PING_TIMEOUT)
                    if ping['success']:
                        return url
                except:
                    pass
    return None


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    print(loop.run_until_complete(get_server('misha')))



import requests, json

SYNO_API_AUTH = '/auth.cgi?api=SYNO.API.Auth&version=2'
SYNO_API_DSM = '/DownloadStation/task.cgi'
SYNO_API_DSM_NAME = 'SYNO.DownloadStation.Task'
SYNO_API_DSM_GET = SYNO_API_DSM + '?api=' + SYNO_API_DSM_NAME + '&version=1'

class API:
    def __init__(self, url, user, password, session):
        self.user = user
        self.password = password
        self.url = url + ('' if url.endswith('/webapi') else '/webapi')

        self.session = session
        self.sid = None
        
        self.http = None
        self.logged = False


    def from_json(js):
        self = API(js['url'], js['user'], js['password'], js['session'])
        self.sid = js['sid']
        self.logged = True
        return self

    def to_json(self):
        return {
            'url': self.url,
            'user': self.user,
            'password': self.password,
            'session': self.session,
            'sid': str(self.sid)
        }
        
    def login(self):
        res = requests.get(
            self.url + SYNO_API_AUTH + '&method=login&format=sid' +
                            '&session=' + self.session +
                            '&account=' + self.user +
                            '&passwd=' + self.password).json()
        try:
            self.sid = res['data']['sid']
        except:
            return None
        self.logged = True
        return self.sid

    def logout(self):
        return requests.get(
            self.url + SYNO_API_AUTH + '&method=logout&' +
                            '&session=' + self.session +
                            '&_sid=' + self.sid)

    def ds_download(self, uri):
        print(uri)
        res = requests.post(
            self.url + SYNO_API_DSM,
            data = {'api': SYNO_API_DSM_NAME, 'version': 1, 'uri': uri, 'method': 'create', '_sid': self.sid}
        ).json()
        print(res)
        return res.get('success', False)

    def _meta_call(self, call, ids = None, addon=""):
        url = self.url + SYNO_API_DSM_GET + '&method=' + call + \
            (('&id=' + ids) if ids else "") + \
            '&_sid=' + self.sid + \
            addon
        return requests.get(
            url
        ).json()

    def ds_pause(self, ids):
        res = self._meta_call('pause', ids)
        return res.get('success', False)

    def ds_resume(self, ids):
        res = self._meta_call('resume', ids)
        return res.get('success', False)

    def ds_delete(self, ids, force=False):
        res = self._meta_call('delete', ids,
                        addon = ('&force_complete=true' if force else ''))
        return res.get('success', False)

    def ds_list(self):
        res = self._meta_call('list')
        if res.get('success', False):
            return res['data']['tasks']
        return []


    def ds_info(self, ids):
        info = self._meta_call('getinfo', ids,
                               addon = '&additional=detail,transfer')
        if info.get('success', False):
            return info['data']['tasks']
        return {}

if __name__ == '__main__':
    api = API('http://192.168.1.29:5000', 'bobuk', 'shmobuk', 'wague-sync')



from setuptools import setup, find_packages
setup(name="syno", version="0.0.3",
      packages=['syno', 'syno.async'],
      requires=['aiohttp', 'asyncio', 'async_timeout'],
      url="http://github.com/bobuk/syno",
      author="Grigory Bakunov",
      author_email='thebobuk@ya.ru',
      description='Synology API wrapper with aiohttp/asyncio')
